<?php

/**
 * @file
 * Contains
 *   \Drupal\webform_submission_files_download\Controller\DefaultController.
 */

namespace Drupal\webform_submission_files_download\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\file\Entity\File;
use Drupal\webform\Entity\WebformSubmission;
use Drupal\webform\Entity\Webform;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * Default controller for the webform_submission_files_download module.
 */
class DefaultController extends ControllerBase {

    public function download_download($webform, $webform_submission) {
        $filename = $webform . '-' . $webform_submission . '-files';
        $webform = Webform::load($webform);
        $webform_submission = WebformSubmission::load($webform_submission);
        $elements_managed_files = $webform->getElementsManagedFiles();
        $fids = [];
        foreach ($elements_managed_files as $k => $element_name) {
            $file_ids = $webform_submission->getElementData($element_name);
            if (!is_array($file_ids) && !empty($file_ids) && is_numeric($file_ids)) {
                $file_ids = [$file_ids];
            }
            $fids = array_merge($fids, $file_ids);
        }

        $filename = $filename . '.zip';
        $archive_path = \Drupal::service('file_system')
            ->saveData('', 'temporary://' . $filename);
        $archive = \Drupal::service('plugin.manager.archiver')
            ->getInstance(['filepath' => $archive_path]);
        $zip_archive = $archive->getArchive();
        $file_entities = File::loadMultiple($fids);

        foreach ($file_entities as $file_entity) {
            $file_path = \Drupal::service("file_system")
                ->realpath($file_entity->getFileUri());
            $zip_archive->addFile($file_path, $file_entity->getFilename());
        }

        $zip_archive->close();

        $headers = [
            'Content-Type' => 'application/force-download',
            'Content-Disposition' => 'attachment;filename="' . $filename . '"',
            //'Content-Length' => $tmp_file->getSize(),
            'Content-Description' => ' File Transfer',
        ];

        return new BinaryFileResponse($archive_path, 200, $headers, TRUE);
    }

}
